# Generated by roxygen2: do not edit by hand

S3method(plot,cptga)
S3method(plot,cptgaisl)
S3method(print,summary.cptga)
S3method(print,summary.cptgaisl)
export(amoc_crossover)
export(amoc_mutation)
export(amoc_population)
export(amoc_selection)
export(arima_bic)
export(arima_bic_order_pq)
export(cpt_dist)
export(cptga)
export(cptgaisl)
export(mutation)
export(random_population)
export(select_tau)
export(selection_linear_rank)
export(ts_sim)
export(uniform_crossover)
exportClasses(cptga)
exportClasses(cptgaisl)
exportMethods(summary)
import(Rcpp)
import(doParallel)
import(foreach)
import(graphics)
import(parallel)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(methods,new)
importFrom(utils,str)
importFrom(utils,tail)
useDynLib(changepointGA)
useDynLib(changepointGA, .registration = TRUE)
