% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\docType{package}
\name{changepointGA-package}
\alias{changepointGA}
\alias{changepointGA-package}
\title{changepointGA: Genetic Algorithms for Changepoint Models}
\description{
Tools for fitting changepoint models with genetic algorithms and Armadillo-backed linear algebra.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mli171/changepointGA}
  \item Report bugs at \url{https://github.com/mli171/changepointGA/issues}
}

}
\author{
\strong{Maintainer}: Mo Li \email{mo.li@louisiana.edu}

Authors:
\itemize{
  \item Mo Li \email{mo.li@louisiana.edu}
  \item QiQi Lu \email{qlu2@vcu.edu}
}

}
\keyword{internal}
